<?php

 /* whatlinkshere.php -- Provides some backlink information
  * Copyright 2004, Evan Prodromou <evan@pigdog.org>.
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

if (!defined('PmWiki')) exit();

define('WHAT_LINKS_HERE_VERSION', 0.1);

# Rules for the two main functions

$DefaultLinkRules = array('barewikiword' => 'LinkRuleBareWikiWord',
			  'groupwikiword' => 'LinkRuleGroupWikiWord',
			  'barefreelink' => 'LinkRuleBareFreeLink',
			  'groupfreelink' => 'LinkRuleGroupFreeLink');

$DefaultDependRules = array('groupheader' => 'DependRuleGroupHeader',
			    'groupfooter' => 'DependRuleGroupFooter',
			    'groupsidebar' => 'DependRuleGroupSidebar',
			    'mainsidebar' => 'DependRuleMainSidebar',
			    'bareinclude' => 'DependRuleBareInclude',
			    'groupinclude' => 'DependRuleGroupInclude',
			    'bareredirect' => 'DependRuleBareRedirect',
			    'groupredirect' => 'DependRuleGroupRedirect');

SDV($LinkRules, $DefaultLinkRules);
SDV($DependRules, $DefaultDependRules);

# A page that shows results of the functions

SDV($WhatLinksHereFmt,
    "<h2 class='wikiaction'>What links here</h2>
     <dl>\$LinksMatchList</dl>
     <h2 class='wikiaction'>Dependencies</h2>
     <dl>\$DependsMatchList</dl>");
SDV($LinksListItemFmt,'<dd><a href="$PageUrl">$Title</a></dd>');
SDV($LinksListGroupFmt,'<dt><a href="$ScriptUrl/$Group">$Group</a> /</dt>');
SDV($LinksListEmptyFmt,'No matches.');

$HandleActions['whatlinkshere'] = 'HandleWhatLinksHere';

function HandleWhatLinksHere($pagename) {
    global $WhatLinksHereFmt, $HandleWhatLinksHereFmt, $PageStartFmt, $PageEndFmt;
    global $CurrentTime, $Author;
    $links = WhatLinksHere($pagename);
    $depends = WhatDependsOn($pagename);
    $GLOBALS['LinksMatchList'] = WhatLinksHereMatchList($links);
    $GLOBALS['DependsMatchList'] = WhatLinksHereMatchList($depends);
    $GLOBALS['LastModified'] = $CurrentTime;
    $GLOBALS['LastModifiedBy'] = @$Author;
    $GLOBALS['HTMLTitle'] = 'What links here';
    $GLOBALS['GCount'] = 0;
    SDV($HandleWhatLinksHereFmt,array(&$PageStartFmt,
				      &$WhatLinksHereFmt,
				      &$PageEndFmt));
    PrintFmt($pagename,$HandleWhatLinksHereFmt);
}

function WhatLinksHereMatchList($pages) {
    global $LinksListItemFmt, $LinksListGroupFmt, $LinksListEmptyFmt;
    if (count($pages) == 0) {
	return $LinksListEmptyFmt;
    } else {
	sort($pages);
	reset($pages);
	$lgroup = '';
	$listtext = '';
	foreach ($pages as $pagename) {
	    $group = FmtPageName('$Group', $pagename);
	    if ($group != $lgroup) {
		$listtext .= FmtPageName($LinksListGroupFmt,$pagename);
		$lgroup = $group;
	    }
	    $listtext .= FmtPageName($LinksListItemFmt, $pagename);
	}
	return $listtext;
    }
}

function WhatLinksHere($pagename) {
    global $LinkRules;
    return FilterOnRules($pagename, $LinkRules);
}

function WhatDependsOn($pagename) {
    global $DependRules;
    return FilterOnRules($pagename, $DependRules);
}

function FilterOnRules($pagename, $rules) {
    global $WikiLibDirs;
    $matches = array();
    $seen = array();
    $wikidirs = $WikiLibDirs; # Have to copy; ReadPage() advances internal pointer!
    foreach ($wikidirs as $wikidir) {
	$dfp = opendir($wikidir);
	if (!$dfp) {
	    continue;
	}
	while (($pagefile=readdir($dfp))!=false) {
	    if ($pagefile[0] == '.') { # ., .., .htaccess, .flock
		continue;
	    }
	    if ($seen[$pagefile]) { # Only check first one
		continue;
	    }
	    $seen[$pagefile] = true;
	    error_log("Got page $pagefile", 0);
	    $page = RetrieveAuthPage($pagefile, "read");
	    # implicit boolean OR. We break on the first rule that returns
	    # true.
	    foreach ($rules as $rule) {
		if ($rule && function_exists($rule)) {
		    if ($rule($pagename, $pagefile, $page)) {
			$matches[] = $pagefile;
			break;
		    }
		}
	    }
	}
	closedir($dfp);
    }
    return $matches;
}
		       
# Default link rules

# Is page 2 the RecentChanges page for page 1's group?

function LinkRuleRecentChanges($pagename1, $pagename2, $page2) {
    return ($pagename2 == FmtPageName("$Group.RecentChanges", $pagename1));
}

# It's a WikiWord, both pages in the same group, bare word found

function LinkRuleBareWikiWord($pagename1, $pagename2, $page2) {
    global $WikiWordPattern, $FragmentPattern;
    $title1 = FmtPageName('$Title', $pagename1);
    return (preg_match("/^$WikiWordPattern$/", $title1) &&
	    FmtPageName('$Group', $pagename1) == FmtPageName('$Group', $pagename2) &&
	    preg_match("/\\b$title1($FragmentPattern)?\\b/", $page2['text']));
}

# It's a WikiWord, group and title found

function LinkRuleGroupWikiWord($pagename1, $pagename2, $page2) {
    global $WikiWordPattern, $FragmentPattern;
    $title1 = FmtPageName('$Title', $pagename1);
    $group1 = FmtPageName('$Group', $pagename1);
    return (preg_match("/^$WikiWordPattern$/", $title1) &&
	    preg_match("/\\b$group1.$title1($FragmentPattern)?\\b/", $page2['text']));
}

# both pages in the same group, bare freelink found

function LinkRuleBareFreeLink($pagename1, $pagename2, $page2) {
    global $FragmentPattern;
    $title1 = FmtPageName('$Title', $pagename1);
    return (FmtPageName('$Group', $pagename1) == FmtPageName('$Group', $pagename2) &&
	    preg_match("/\{\{$title1($FragmentPattern)?.*?\}\}/", $page2['text']));
}

# group freelink found

function LinkRuleGroupFreeLink($pagename1, $pagename2, $page2) {
    global $FragmentPattern;
    $title1 = FmtPageName('$Title', $pagename1);
    $group1 = FmtPageName('$Group', $pagename1);
    return (preg_match("/\{\{$group1.$title1($FragmentPattern)?.*?\}\}/", $page2['text']));
}

# Default depend rules

# Is page 1 the group header for page 2?

function DependRuleGroupHeader($pagename1, $pagename2, $page2) {
    global $GroupHeaderFmt;
    return ($pagename1 == FmtPageName($GroupHeaderFmt, $pagename2) &&
	    !strstr($page2['text'], "[[nogroupheader]]"));
}

# Is page 1 the group footer for page 2?

function DependRuleGroupFooter($pagename1, $pagename2, $page2) {
    global $GroupFooterFmt;
    return ($pagename1 == FmtPageName($GroupFooterFmt, $pagename2) &&
	    !strstr($page2['text'], "[[nogroupfooter]]"));
}

# Is page 1 the group sidebar for page 2?

function DependRuleGroupSidebar($pagename1, $pagename2, $page2) {
    return ($pagename1 == FmtPageName("$Group.SideBar", $pagename2));
}

# Is page 1 the main sidebar?

function DependRuleMainSidebar($pagename1, $pagename2, $page2) {
    return ($pagename1 == 'Main.SideBar');
}

# Are they in the same group, and page 2 includes part or all of page 1?

function DependRuleBareInclude($pagename1, $pagename2, $page2) {
    global $FragmentPattern;
    $title1 = FmtPageName('$Title', $pagename1);
    return (FmtPageName('$Group', $pagename1) == FmtPageName('$Group', $pagename2) &&
	    preg_match("/\\[\\[include:$title1($FragmentPattern)?\\]\\]/",$page2['text']));
}

# Does page 2 include part or all of page 1, with a full name?

function DependRuleGroupInclude($pagename1, $pagename2, $page2) {
    global $FragmentPattern;
    $title1 = FmtPageName('$Title', $pagename1);
    $group1 = FmtPageName('$Group', $pagename1);
    return preg_match("/\\[\\[include:$group1\[\\.\\/\]$title1($FragmentPattern)?\\]\\]/",$page2['text']);
}

# Does page 2 redirect to page 1?

function DependRuleBareRedirect($pagename1, $pagename2, $page2) {
    global $FragmentPattern;
    $title1 = FmtPageName('$Title', $pagename1);
    return (FmtPageName('$Group', $pagename1) == FmtPageName('$Group', $pagename2) && 
	    preg_match("/\\[\\[redirect:$title1($FragmentPattern)?\\]\\]/",$page2['text']));
}
# Does page 2 redirect to page 1?

function DependRuleGroupRedirect($pagename1, $pagename2, $page2) {
    global $FragmentPattern;
    $group1 = FmtPageName('$Group', $pagename1);
    $title1 = FmtPageName('$Title', $pagename1);
    return preg_match("/\\[\\[redirect:$group1\[\\/.\]$title1($FragmentPattern)?\\]\\]/",$page2['text']);
}


?>
